@@ -1,6 +1,6 @@
 /*
  * Normaliz
- * Copyright (C) 2007-2019  Winfried Bruns, Bogdan Ichim, Christof Soeger
+ * Copyright (C) 2007-2022  W. Bruns, B. Ichim, Ch. Soeger, U. v. d. Ohe
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * As an exception, when this program is distributed through (i) the App Store
  * by Apple Inc.; (ii) the Mac App Store by Apple Inc.; or (iii) Google Play
@@ -21,7 +21,7 @@
  * terms of service.
  */
 
-#include <stdlib.h>
+#include <cstdlib>
 #include <vector>
 #include <list>
 #include <string>
@@ -29,6 +29,7 @@
 #include <fstream>
 #include <algorithm>
 #include <csignal>
+#include <memory>
 using namespace std;
 
 #ifdef NMZ_GPERF
@@ -38,61 +39,30 @@ using namespace std;
 #include "libnormaliz/integer.h"
 #include "libnormaliz/cone.h"
 #include "libnormaliz/output.h"
+#include "libnormaliz/input.h"
+#include "libnormaliz/options.h"
+#include "libnormaliz/chunk.h"
 
 using namespace libnormaliz;
-#include "input.cpp"
-#include "options.cpp"
-
-#ifndef STRINGIFY
-#define STRINGIFYx(Token) #Token
-#define STRINGIFY(Token) STRINGIFYx(Token)
-#endif
 
 long CCCCCCC = 0;
 
-void printCopying() {
-    cout << "Copyright (C) 2007-2019  The Normaliz Team, University of Osnabrueck." << endl
-         << "This program comes with ABSOLUTELY NO WARRANTY; This is free software," << endl
-         << "and you are welcome to redistribute it under certain conditions;" << endl
-         << "See COPYING for details." << endl;
-}
-
-void printVersion() {
-    cout << "Normaliz " << string(STRINGIFY(NMZ_VERSION)) << endl;
-    printCopying();
-}
-
 void printHeader() {
-    cout << "                                                    \\.....|" << endl;
+    cout << "                                                     \\.....|" << endl;
 #ifdef NMZ_DEVELOP
     cout << "                 Normaliz DEVELOPMENT "
-         << "               \\....|" << endl;
+         << "                \\....|" << endl;
 #else
-    cout << "                    Normaliz " << string(STRINGIFY(NMZ_VERSION) "           ", 11) << "             \\....|" << endl;
+    cout << "                    Normaliz " << string(STRINGIFY(NMZ_VERSION) "           ", 11) << "              \\....|"
+         << endl;
 #endif
-    cout << "                                                      \\...|" << endl;
-    cout << "     (C) The Normaliz Team, University of Osnabrueck   \\..|" << endl;
-    cout << "                      June 2020                         \\.|" << endl;
-    cout << "                                                         \\|" << endl;
-    bool with_optional_packages = false;
-    string optional_packages;
-#ifdef NMZ_COCOA
-    with_optional_packages = true;
-    optional_packages += " CoCoALib";
-#endif
-#ifdef NMZ_FLINT
-#ifndef ENFNORMALIZ
-    with_optional_packages = true;
-    optional_packages += " Flint";
-#endif
-#endif
-#ifdef ENFNORMALIZ
-    with_optional_packages = true;
-    optional_packages += " Flint antic arb e-antic";
-#endif
-
-    if (with_optional_packages) {
-        cout << "------------------------------------------------------------" << endl;
+    cout << "                                                       \\...|" << endl;
+    cout << "     (C) The Normaliz Team, University of Osnabrueck    \\..|" << endl;
+    cout << "                     August 2022                         \\.|" << endl;
+    cout << "                                                          \\|" << endl;
+    string optional_packages = package_string();
+    if (optional_packages.size() > 0) {
+        cout << "-------------------------------------------------------------" << endl;
         cout << "with package(s)" << optional_packages << endl;
     }
 }
@@ -100,24 +70,15 @@ void printHeader() {
 void printHelp(char* command) {
     cout << "Usage: " << command << " [options] PROJECT" << endl;
     cout << "  runs normaliz on PROJECT.in" << endl;
-    cout << "Options:" << endl;
-    cout << "  -S\tcompute sublattice" << endl;
+    cout << "Computation goals with short options (selection):" << endl;
     cout << "  -s\tcompute support hyperplanes" << endl;
-    cout << "  -t\tcompute triangulation" << endl;
     cout << "  -v\tcompute multiplicity" << endl;
-    cout << "  -v\tcompute volume" << endl;
-    cout << "  -F\tcompute volume by decent in the face lattice" << endl;
-    cout << "  -n\tcompute Hilbert basis and multiplicity (needs full triangulation)" << endl;
+    cout << "  -V\tcompute volume" << endl;
     cout << "  -N\tcompute Hilbert basis (with partial triangulation)" << endl;
     cout << "  -w\tcheck for integrally closed and compute witness if not" << endl;
-    cout << "  -q\tcompute Hilbert (quasi-)polynomial" << endl;
-    cout << "  -p\tcompute Hilbert (quasi-)polynomial and degree 1 elements" << endl;
-    cout << "  -h\tcompute Hilbert basis and Hilbert polynomial (default)" << endl;
+    cout << "  -q\tcompute Hilbert series" << endl;
     cout << "  -1\tcompute degree 1 elements" << endl;
-    cout << "  -y\tcompute Stanley decomposition (output in file .dec)" << endl;
-    cout << "  -C\tcompute class group (default)" << endl;
     cout << "  -T\tcompute triangulation  (output in file .tri)" << endl;
-    cout << "  -D\tcompute cone decomposition (includes -T)" << endl;
     cout << "  -H\tcompute integer hull" << endl;
     cout << "  -M\tcompute module generators over original monoid" << endl;
     cout << "  -E\tcompute weighted Ehrhart series" << endl;
@@ -126,20 +87,27 @@ void printHelp(char* command) {
     cout << "  -G\tcheck Gorenstein" << endl;
 
     cout << endl;
-    cout << "  -d\tcomputation mode: dual" << endl;
-    cout << "  -P\tcomputation mode: primal" << endl;
-    cout << "  -j\tcomputation mode: project-and-lift" << endl;
-    cout << "  -J\tcomputation mode: project-and-lift with floating point arithmetic" << endl;
-    cout << "  -r\tcomputation mode: approximate" << endl;
-    cout << "  -b\tcomputation mode: bottom decomposition" << endl;
-    cout << "  -o\tcomputation mode: no bottom decomposition" << endl;
-    cout << "  -k\tcomputation mode: keep order" << endl;
-    cout << "  -Y\tcomputation mode: symmetrization" << endl;
+    cout << "Algorithmic variants with short options (slection):" << endl;
+    cout << "  -d\t dual ode (includes Hilbert basis, unless combined with -1)" << endl;
+    cout << "  -j\t project-and-lift" << endl;
+    cout << "  -J\t project-and-lift with floating point arithmetic" << endl;
+    cout << "  -k\t keep order" << endl;
+    cout << "  -Y\t symmetrization" << endl;
+    cout << "  -F\t multiplicity/volume by decent in the face lattice" << endl;
 
     cout << endl;
+    cout << "For computation goals and variants not in the lists above use" << endl;
+    // cout << endl;
     cout << "      --<PROP>     compute the ConeProperty <PROP>" << endl;
+    // cout << endl;
+    cout << "see doc/Normaliz.pdf or doc/NmzShortRef.pdf. Selection:" << endl;
+    // cout << endl;
+    cout << "      Automorphisms, EuclideanAutomorphisms, RationalA..., CombinatorialA..." << endl;
+    cout << "      EhrhartSeries, LatticePoints, NumberLatticePoints" << endl;
+    cout << "      FaceLattice, FVector, Incidence" << endl;
 
     cout << endl;
+    cout << "Output and execution:" << endl;
     cout << "  -f, --files      write the files .out .gen .inv .cst" << endl;
     cout << "  -a, --all-files  write all output files (except  .dec .tri .typ)" << endl;
     cout << "      --<SUFFIX>   write the file .<SUFFIX> where <SUFFIX> can be one of" << endl;
@@ -152,14 +120,14 @@ void printHelp(char* command) {
     cout << "  -x=<T>           limit the number of threads to <T>" << endl;
     cout << "  --OutputDir=<path> set a path for the output files (relative to current directory)" << endl;
     cout << "  -?, --help       print this help text and exit" << endl;
-    cout << "  -c, --verbose    verbose (prints control data)" << endl;
+    cout << "  -c, --verbose    verbose (prints log data on terminal)" << endl;
     cout << "      --version    print version info and exit" << endl;
     cout << endl;
     cout << "Please report bugs to <normaliz@uos.de> or directly to our issue tracker:" << endl;
     cout << "https://github.com/Normaliz/Normaliz/issues" << endl;
 }
 
-int process_data(OptionsHandler& options, const string& command_line, renf_class& number_field);
+int process_data(OptionsHandler& options, const string& command_line);
 
 //---------------------------------------------------------------------------
 
@@ -167,6 +135,8 @@ int main(int argc, char* argv[]) {
 #ifdef NMZ_GPERF
     ProfilerStart("normaliz.prof");
 #endif
+    
+    StartGlobalTime();
 
     /*cout << "Before AAA" << endl;
     renf_class K("a^2 - 5", "a", "2.0 +/- 1.0");
@@ -179,7 +149,6 @@ int main(int argc, char* argv[]) {
     signal(SIGINT, &interrupt_signal_handler);
 
     // read command line options
-
     OptionsHandler options;
 
     string command_line;
@@ -198,9 +167,11 @@ int main(int argc, char* argv[]) {
         printHeader();
     }
 
-    renf_class number_field;  // is bool without e-antic
+    process_data(options, command_line);
 
-    process_data(options, command_line, number_field);
+    if (verbose && GMP_hyp + GMP_scal_prod + GMP_mat > 0)
+        verboseOutput() << "GMP transitions: matrices " << GMP_mat << " hyperplanes " << GMP_hyp << " vector operations "
+                        << GMP_scal_prod << endl;
 
     if (nmz_interrupted)
         exit(10);
@@ -208,6 +179,8 @@ int main(int argc, char* argv[]) {
 #ifdef NMZ_GPERF
     ProfilerStop();
 #endif
+    
+    MeasureGlobalTime(verbose);
 
     exit(0);
 }
@@ -216,13 +189,21 @@ int main(int argc, char* argv[]) {
 
 template <typename ConeType, typename InputNumberType>
 void compute_and_output(OptionsHandler& options,
-                        const map<Type::InputType, vector<vector<InputNumberType> > >& input,
+                        const InputMap<InputNumberType>& input,
                         const map<NumParam::Param, long>& num_param_input,
-                        const string& polynomial,
-                        renf_class& number_field,
-                        const map<Type::InputType, vector<vector<InputNumberType> > >& add_input) {
+                        map<PolyParam::Param, vector<string> >& poly_param_input,
+                        renf_class_shared number_field_ref,
+                        InputMap<InputNumberType>& add_input) {
     Output<ConeType> Out;  // all the information relevant for output is collected in this object
 
+    // const
+    renf_class_shared number_field =
+#ifdef ENFNORMALIZ
+        number_field_ref.get();
+#else
+        number_field_ref;
+#endif
+
     options.applyOutputOptions(Out);
 
     options.activateDefaultMode();
@@ -230,12 +211,13 @@ void compute_and_output(OptionsHandler& options,
     Out.set_lattice_ideal_input(input.count(Type::lattice_ideal) > 0);
 
     Cone<ConeType> MyCone = Cone<ConeType>(input);
-    MyCone.setPolynomial(polynomial);
+    MyCone.setPolyParams(poly_param_input);
     MyCone.setNumericalParams(num_param_input);
     /*MyCone.setNrCoeffQuasiPol(nr_coeff_quasipol);
     MyCone.setExpansionDegree(expansion_degree);
     MyCone.setFaceCodimBound(face_codim_bound);*/
-    MyCone.setRenf(&number_field);
+    MyCone.setRenf(number_field);
+    MyCone.setProjectName(options.getProjectName());
     try {
         MyCone.compute(options.getToCompute());
         if (add_input.size() > 0) {
@@ -254,7 +236,7 @@ void compute_and_output(OptionsHandler& options,
         std::cout << "Writing only available data." << endl;
     }
     Out.setCone(MyCone);
-    Out.set_renf(&number_field);
+    Out.set_renf(number_field);
 
     signal(SIGINT, SIG_DFL);
 
@@ -265,7 +247,7 @@ void compute_and_output(OptionsHandler& options,
         options.applyOutputOptions(IntHullOut);
         IntHullOut.set_name(options.getProjectName() + ".IntHull");
         IntHullOut.setCone(MyCone.getIntegerHullCone());
-        IntHullOut.set_renf(&number_field, true);
+        IntHullOut.set_renf(number_field, true);
         IntHullOut.write_files();
     }
 
@@ -274,7 +256,7 @@ void compute_and_output(OptionsHandler& options,
         options.applyOutputOptions(ProjOut);
         ProjOut.set_name(options.getProjectName() + ".ProjectCone");
         ProjOut.setCone(MyCone.getProjectCone());
-        ProjOut.set_renf(&number_field);
+        ProjOut.set_renf(number_field);
         ProjOut.write_files();
     }
 
@@ -292,9 +274,9 @@ void compute_and_output(OptionsHandler& options,
 //---------------------------------------------------------------------------
 // for testing only, not really useful in Normaliz
 template <typename InputNumberType>
-map<Type::InputType, vector<vector<InputNumberType> > > extract_additional_input(
-    map<Type::InputType, vector<vector<InputNumberType> > >& input) {
-    map<Type::InputType, vector<vector<InputNumberType> > > add_input;
+InputMap<InputNumberType> extract_additional_input(
+    InputMap<InputNumberType>& input) {
+    InputMap<InputNumberType> add_input;
     size_t nr_add_input = 0;
     auto M = input.find(Type::add_inequalities);
     if (M != input.end()) {
@@ -342,13 +324,23 @@ map<Type::InputType, vector<vector<InputNumberType> > > extract_additional_input
 }
 //---------------------------------------------------------------------------
 
-int process_data(OptionsHandler& options, const string& command_line, renf_class& number_field) {
+int process_data(OptionsHandler& options, const string& command_line) {
     try {
-        if (options.getProjectName() == "") {
+        if (options.getProjectName() == "" && !options.isUseChunk()) {
             cerr << "ERROR: No project name set!" << endl;
             exit(1);
         }
 
+        if (options.isUseChunk()) {
+            chunk();
+            exit(0);
+        }
+
+        if (options.isUseAddChunks()) {
+            add_chunks(options.getProjectName());
+            exit(0);
+        }
+
         string name_in = options.getProjectName() + ".in";
         const char* file_in = name_in.c_str();
         ifstream in;
@@ -358,18 +350,21 @@ int process_data(OptionsHandler& options, const string& command_line, renf_class
             exit(1);
         }
 
-        string polynomial = "";  // these are default values
+        string polynomial;
+        vector<string> polynomial_equations;
         /*long nr_coeff_quasipol=-1;
         long expansion_degree=-1;
         long face_codim_bound=-1;*/
 
-        map<Type::InputType, vector<vector<mpq_class> > > input, add_input;
-        map<Type::InputType, vector<vector<renf_elem_class> > > renf_input, renf_add_input;
+        InputMap<mpq_class> input, add_input;
+        InputMap<renf_elem_class> renf_input, renf_add_input;
         map<NumParam::Param, long> num_param_input;
+        map<PolyParam::Param, vector<string> > poly_param_input;
         bool renf_read = false;
+        renf_class_shared number_field;
 
         try {
-            input = readNormalizInput<mpq_class>(in, options, num_param_input, polynomial, number_field);
+            input = readNormalizInput<mpq_class>(in, options, num_param_input, poly_param_input,  number_field);
             if (nmz_interrupted)
                 exit(10);
         }
@@ -380,7 +375,7 @@ int process_data(OptionsHandler& options, const string& command_line, renf_class
 
             in.close();
             in.open(file_in, ifstream::in);
-            renf_input = readNormalizInput<renf_elem_class>(in, options, num_param_input, polynomial, number_field);
+            renf_input = readNormalizInput<renf_elem_class>(in, options, num_param_input, poly_param_input, number_field);
             if (nmz_interrupted)
                 exit(10);
             renf_read = true;
@@ -394,9 +389,15 @@ int process_data(OptionsHandler& options, const string& command_line, renf_class
         in.close();
 
         if (verbose) {
-            cout << "------------------------------------------------------------" << endl;
-            cout << "Command line: " << command_line << endl;
-            cout << "Compute: " << options.getToCompute() << endl;
+            verboseOutput() << "-------------------------------------------------------------" << endl;
+            verboseOutput() << "Command line: " << command_line << endl;
+            verboseOutput() << "Compute: ";
+            if (options.getToCompute().none())
+                verboseOutput() << "No computation goal set, using defaults given input" << endl;
+            else
+                verboseOutput() << options.getToCompute() << endl;
+            for(auto& P: num_param_input)
+                verboseOutput() << numpar_to_string(P.first) << " = " << P.second << endl;
         }
 
         if (renf_read) {
@@ -405,18 +406,18 @@ int process_data(OptionsHandler& options, const string& command_line, renf_class
             // if(options.getToCompute().test(ConeProperty::Dynamic))
             renf_add_input = extract_additional_input<renf_elem_class>(renf_input);
 
-            compute_and_output<renf_elem_class>(options, renf_input, num_param_input, polynomial, number_field, renf_add_input);
+            compute_and_output<renf_elem_class>(options, renf_input, num_param_input, poly_param_input, number_field, renf_add_input);
         }
         else {
             if (options.isUseLongLong()) {
                 // if(options.getToCompute().test(ConeProperty::Dynamic))
                 add_input = extract_additional_input<mpq_class>(input);
-                compute_and_output<long long>(options, input, num_param_input, polynomial, number_field, add_input);
+                compute_and_output<long long>(options, input, num_param_input, poly_param_input, number_field, add_input);
             }
             else {
                 // if(options.getToCompute().test(ConeProperty::Dynamic))
                 add_input = extract_additional_input<mpq_class>(input);
-                compute_and_output<mpz_class>(options, input, num_param_input, polynomial, number_field, add_input);
+                compute_and_output<mpz_class>(options, input, num_param_input, poly_param_input, number_field, add_input);
             }
         }
 
